<?php

namespace App\Http\Controllers;

use App\Events\SendEmailEvent;
use App\Http\Requests\UserRequest\ChangePasswordRequest;
 


use App\Http\Requests\UserRequest\CreateUserRequest;
use App\Http\Requests\UserRequest\EditUserRequest;
use App\Http\Requests\UserRequest\UpdatePasswordRequest;
use App\Models\TenderCompany;
use App\Repositories\UserRepository;
use Illuminate\Http\Request;
 

class UserController extends Controller
{ 
    private $UserRepository;
    public function __construct(UserRepository $UserRepository)
    { 
        $this->middleware('permission:user.list',['only'=>['index']]); 
        $this->middleware('permission:user.create', ['only' => ['create']]);
        $this->middleware('permission:user.update', ['only' => ['edit','update']]);
        $this->middleware('permission:user.delete', ['only' => ['destroy']]);
        $this->middleware('permission:user.show', ['only' => ['show']]);
    

        $this->middleware('auth')->except(['storeCompany', 'createCompany']);;  
        $this->UserRepository = $UserRepository;
    }
    /**
     * Display a listing of the resource.
     *
     * @return \Illuminate\Http\Response
     */
    public function index()
    {
        $users = $this->UserRepository->getUsers(); 
        return view('user.index',compact('users'));
    }
    
    /**
     * Show the form for creating a new resource.
     *
     * @return \Illuminate\Http\Response
     */
    public function create()
    {
        $roles = $this->UserRepository->getRoles();
       
        return view('user.create', compact('roles'));
    }

    /**
     * Store a newly created resource in storage.
     *
     * @param  \Illuminate\Http\Request  $request
     * @return \Illuminate\Http\Response
     */
    public function store(CreateUserRequest $request)
    {  
        
        $user = $this->UserRepository->createUser($request);
       
        if ($user instanceof \Exception) {
            return redirect('company')->with('error', $user->getMessage());
        }
  
        return redirect(url("company"))->with('success', 'Company has been Created');
    }
    public function createCompany() 
    {   
        $roles = $this->UserRepository->getRoles();
        return view('user.register', compact('roles'));
    }
 
    public function storeCompany(CreateUserRequest $request)
    {    
         

        $user = $this->UserRepository->createUser($request);
      
        if ($user instanceof \Exception) {
            return redirect()->back()->with('error', $user->getMessage());
        }
        
        return redirect('login')->with('success', 'Account created succesfully. please login to proceed');
    }
 
    public function show($id )
    { 
         
        $company_id = decrypt($id); 
          
        $user = $this->UserRepository->getUserById($company_id); 
        if ($user instanceof \Exception) { 
            return response()->json(['status' => false, 'msg' => $user->getMessage()]);
        }
        $image = url('/site_images/users/'.$user->image);
         
        return response()->json(['status' => true, 'msg' => 'Success', 'company' =>$user, 'image' => $image]);
        
    }

    /**
     * Show the form for editing the specified resource.
     *
     * @param  int  $id
     * @return \Illuminate\Http\Response
     */
    public function edit($id)
    {
        
        $id = decrypt($id);
        
        $roles = $this->UserRepository->getRoles();
        $user = $this->UserRepository->getUserById($id); 
        if ($user instanceof \Exception) {
            return redirect('company')->with('error', $user->getMessage());
        }
        return view('user.edit', compact('user', 'roles'));
    }

    /**
     * Update the specified resource in storage.
     *
     * @param  \Illuminate\Http\Request  $request
     * @param  int  $id
     * @return \Illuminate\Http\Response
     */
    public function update(EditUserRequest $request, $id)
    {    
        $id = decrypt($id);
        $user = $this->UserRepository->updateUser($request,$id); 
        
        if ($user instanceof \Exception) {
            return redirect('company')->with('error', $user->getMessage());
        }
        return redirect('company')->with('success', 'Company has been updated');
    }

    /**
     * Remove the specified resource from storage.
     *
     * @param  int  $id
     * @return \Illuminate\Http\Response
     */
    public function destroy($id)
    {
        $id = decrypt($id);
        $user =  $this->UserRepository->deleteUser($id);
        
        if ($user instanceof \Exception) {
            return response()->json(['status' => false, 'error' => $user->getMessage()]);
        }
        return response()->json(['status' => true, 'success' => 'Company has been deleted successfully']);
    }
    public function updatePssword(UpdatePasswordRequest $request)
    {
        $id = decrypt($request->id);
        $request['id'] = $id;
        $password =  $this->UserRepository->updatePssword($request); 
        if ($password instanceof \Exception) {
            return response()->json(['status' => false, 'error' => $password->getMessage()]);
        }
        return response()->json(['status' => true, 'success' => 'Password has been changed successfully']);
    }
    public function updateStatus($id)
    {
        
       $id = decrypt($id);
        $status =  $this->UserRepository->updateStatus($id); 
        if ($status instanceof \Exception) {
            return response()->json(['status' => false, 'error' => $status->getMessage()]);
        }
        if($status){
            return response()->json(['status' => true , 'msg' => $status]);
        }
        return response()->json(['status' => false, 'error' => "Something want wrong"]);
    }

    public function changePassword()
    { 
        return view('user.change-password'); 
    }
    public function saveChangePassword( ChangePasswordRequest $request )
    { 
        $id = decrypt($request->id); 
        $request['id'] = $id;
        $password =  $this->UserRepository->changePassword($request);
       
        if ($password instanceof \Exception) {
            return response()->json(['status' => false, 'error' => $password->getMessage()]);
        }
        if(!$password){
            return response()->json(['status' => false, 'msg' => "Invalid old password"]);  
        }else if($password){
            return response()->json(['status' => true, 'msg' => "Password changed!"]); 
        } 
    }
    
    public function editProfile($id)
    { 
        $id = decrypt($id);
        $user = $this->UserRepository->getUserById($id);
        if ($user instanceof \Exception) {
            return redirect('user')->with('error', $user->getMessage());
        }
        return view('user.edit-profile',compact('user'));
    }
    public function updateProfile(EditUserRequest $request, $id)
    { 
       
        $id = decrypt($id);
        $user = $this->UserRepository->updateUserProfile($request,$id); 
        if ($user instanceof \Exception) {
            return redirect('/dashboard')->with('error', $user->getMessage());
        }
        return redirect('/dashboard')->with('success', 'Profile updated');
    }
}